﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;
using FolderZipper;
using System.Windows.Forms;
using Microsoft.Win32;

namespace SupCom2_Mod_Manager
{
    public class SC2MMData
    {
        private static string mydirectory = "\\SC2MM\\"; //Sets the root SC2MM Application Data directory.

        public static class ModList //unused
        {
            static List<Mod> m_list;
            
            static ModList()
            {
                m_list = new List<Mod>();
            }

            public static void Add(Mod value)
            {
                m_list.Add(value);
            }
        }

        public class Mod
        {
            
            public string FileRef { get; set; } //File Path
            public string DirRef { get; set; } //Folder Path
            public bool MultiMod { get; set; } //Multi Mod Status
            public string ModName { get; set; } //Mod Name
            public string Author { get; set; } //Mod Authors
            public string ShortDesc { get; set; } //Mod Short Description
            public string LongDesc { get; set; } //Mod Long Description
            public string ID { get; set; } //Mod ID
            public double Version { get; set; } //Mod Version
            public bool Enabled { get; set; } //Enabled State
            public int Priority { get; set; } //Priority
            public string ModURL { get; set; } // Mod URL
            public string ReqModName { get; set; } //Required Mod Name
            public string ReqModID { get; set; } //Required Mod ID
            public string ReqModVer { get; set; } //Required Mod Version
            public string Conflicts { get; set; } //Required Mod Version
            public bool Patch { get; set; } //Mod is patch
            public string PatchVer { get; set; } //Version of mod this patch updates.

            private static string p_appdata = GetAppDataPath();
            private static string p_moddir = GetModsPath();
            private static string p_tempdir = GetTempPath();
            private static string p_scddir = GetSCDPath();

            //Adding a new mod
            public static Mod AddMod(FileInfo modfile)
            {
                Mod c_mod = new Mod
                {
                    Enabled = false,
                };
                bool Multi;
                string f_modname = modfile.Name.Replace(".scd", "");
                string f_cfg = p_tempdir + f_modname + "\\mod.cfg";

                if (Directory.Exists(p_tempdir + f_modname))
                    Directory.Delete(p_tempdir + f_modname, true);

                FolderZipper.ZipUtil.UnZipFiles(modfile.FullName, p_tempdir + f_modname, "", false);
                if (!File.Exists(f_cfg))
                {
                    c_mod.Priority = -1; //Catch for an invalid or missing configuration file.
                }
                else  //read config file values into mod object and give it a folder
                {

                    c_mod.Priority = 0;
                    c_mod.ModName = ReadCFG(f_cfg,"$Name ");
                    if (c_mod.ModName.StartsWith("No Data for "))
                    {
                        c_mod.Priority = -1;
                        MessageBox.Show("No Name!", "Error");
                    }
                    c_mod.Author = ReadCFG(f_cfg, "$Author ");
                    c_mod.ShortDesc = ReadCFG(f_cfg, "$ShortDesc ");
                    c_mod.LongDesc = ReadCFG(f_cfg, "$LongDesc ");
                    c_mod.ID = ReadCFG(f_cfg, "$ID ");

                    bool.TryParse(ReadCFG(f_cfg, "$MultiMod "),out Multi);
                    c_mod.MultiMod = Multi;

                    if (c_mod.ID.StartsWith("No Data for "))
                    {
                        c_mod.Priority = -1;
                        MessageBox.Show("No ID!","Error");
                    }
                    if (!IsAlphaNumeric(c_mod.ID))
                    {
                        c_mod.Priority = -1;
                        MessageBox.Show("ID is not alphanumeric!", "Error");
                    }

                    c_mod.Version = double.Parse(ReadCFG(f_cfg, "$Ver "));

                    c_mod.ModURL = ReadCFG(f_cfg, "$ModURL ");
                    if (c_mod.ModURL.Contains("No Data ") == false & c_mod.ModURL != "")
                    {
                        if (!IsAlphaNumeric(c_mod.ModURL))
                        {
                            c_mod.Priority = -1;
                            MessageBox.Show("Mod URL is not a valid thread ID", "Error");
                        }
                    }
                    c_mod.ReqModName = ReadCFG(f_cfg, "$ReqModName ");
                    c_mod.ReqModID = ReadCFG(f_cfg, "$ReqModID ");

                    c_mod.ReqModVer = ReadCFG(f_cfg, "$ReqModVer ");

                    c_mod.Conflicts = ReadCFG(f_cfg, "$ConflictMod ");

                    bool patch;
                    bool.TryParse(ReadCFG(f_cfg, "$Patch "),out patch);
                    c_mod.Patch = patch;

                    c_mod.PatchVer = ReadCFG(f_cfg, "$PatchVer ");

                    c_mod.DirRef = c_mod.ID + "\\";
                    c_mod.FileRef = modfile.Name;
                    if (Directory.Exists(p_tempdir + f_modname))
                    {
                        if (Directory.Exists(p_tempdir + c_mod.DirRef))
                        Directory.Delete(p_tempdir + c_mod.DirRef,true);

                        if (File.Exists(f_cfg))
                        {
                            File.Delete(f_cfg);
                        }
                        Directory.Move(p_tempdir + f_modname, p_tempdir + c_mod.DirRef);
                    }
                }

                return c_mod;
            }

            public static Mod FinalizePatchMod(Mod NewMod, Mod OldMod)
            {
                if (!NewMod.MultiMod)
                {
                    MessageBox.Show("Patches cannot be applied to this type of mod", "Error");
                    //Need to fix .scd recreation before being able to replace non-$MultiMods.
                    return OldMod;
                }
                OldMod.Author = NewMod.Author;
                OldMod.FileRef = NewMod.FileRef;
                OldMod.LongDesc = NewMod.LongDesc;
                OldMod.ModName = NewMod.ModName;
                OldMod.ModURL = NewMod.ModURL;
                OldMod.MultiMod = NewMod.MultiMod;
                OldMod.ReqModID = NewMod.ReqModID;
                OldMod.ReqModName = NewMod.ReqModName;
                OldMod.Version = NewMod.Version;

                List<DirectoryInfo> PatchDirs = new List<DirectoryInfo> { 
                    new DirectoryInfo(GetDescPath() + OldMod.DirRef),
                    new DirectoryInfo(GetSCDPath() + OldMod.DirRef),
                    new DirectoryInfo(GetModsPath() + OldMod.DirRef)
                };

                DirectoryInfo NewModTempDir = new DirectoryInfo(GetTempPath() + NewMod.DirRef);

                foreach (FileInfo newfile in NewModTempDir.GetFiles())
                {

                    if (newfile.Name.Contains(".rtf"))
                    {
                        if (File.Exists(PatchDirs[0] + newfile.Name)) //TODO: Fix orphaned .rtf file that can be left here if the name changes
                        {
                            File.Delete(PatchDirs[0] + newfile.Name);
                            newfile.MoveTo(PatchDirs[0] + newfile.Name);
                        }
                        else
                        {
                            newfile.MoveTo(PatchDirs[0] + newfile.Name); 
                        }

                    }

                    if (newfile.Name.Contains(".scd"))
                    {
                        if (File.Exists(PatchDirs[1] + newfile.Name))
                        {
                            File.Delete(PatchDirs[1] + newfile.Name);
                            newfile.MoveTo(PatchDirs[1] + newfile.Name);
                        }
                        else
                        {
                            newfile.MoveTo(PatchDirs[1] + newfile.Name);
                        }

                    }
                }
                
                
                Directory.Delete(GetTempPath() + NewMod.DirRef,true);
                return OldMod;
            }

            public static void FinalizeMod(FileInfo fm_file,Mod fm_mod)
            {
                string ToMove_desc = GetModsPath() + fm_mod.DirRef + fm_mod.LongDesc;
                string desc_path = GetDescPath() + fm_mod.DirRef;
                if (Directory.Exists(p_moddir + fm_mod.DirRef))
                    Directory.Delete(p_moddir + fm_mod.DirRef, true);

                if (Directory.Exists(p_tempdir + fm_mod.DirRef) & !Directory.Exists(p_moddir + fm_mod.DirRef))
                {
                    Directory.Move(p_tempdir + fm_mod.DirRef, p_moddir + fm_mod.DirRef);
                    if (!Directory.Exists(p_scddir + fm_mod.DirRef))
                    {
                        Directory.CreateDirectory(p_scddir + fm_mod.DirRef);
                    }

                    if (File.Exists(ToMove_desc))
                    {
                        if (!Directory.Exists(desc_path))
                        {
                            Directory.CreateDirectory(desc_path);
                        }
                        if (File.Exists(desc_path + fm_mod.LongDesc))
                        {
                            File.Delete(desc_path + fm_mod.LongDesc);
                        }
                            File.Move(ToMove_desc, desc_path + fm_mod.LongDesc);
                    }
                    if (!fm_mod.MultiMod)
                    {
                        File.Copy(fm_file.FullName, p_scddir + fm_mod.DirRef + fm_mod.FileRef, true);
                    }
                    else
                    { 
                        foreach (string f in Directory.GetFiles(GetModsPath() + fm_mod.DirRef))
                        {
                            FileInfo file = new FileInfo(f);
                            if (file.Name.Contains(".scd"))
                            {
                                File.Copy(f, p_scddir + fm_mod.DirRef + file.Name, true);
                            }
                        }
                    }
                    //FolderZipper.ZipUtil.ZipFiles(GetModsPath() + fm_mod.DirRef, GetSCDPath() + fm_mod.DirRef + fm_mod.FileRef, "");  //Note: Doesn't work!  (Zip util keeps a hold on the original files, mod can't be removed or modified...)
                }
                else
                    MessageBox.Show("A problem occured writing to the temporary directory\n Mod may not work properly.", "Error");
            }

            private static string ReadCFG(string path,string m_value)
            {
                string s_valuetext = "No Data for " + m_value;
                string line = "";
                StreamReader sr = new StreamReader(path);
                while (!sr.EndOfStream)
                {
                    line = sr.ReadLine();
                    if(line.TrimStart(' ').StartsWith(m_value) & line.Contains("<<") & line.Contains(">>"))
                    {
                        line = line.Remove(0, line.IndexOf("<<") + 2);
                        line = line.Remove(line.IndexOf(">>"));
                        s_valuetext = line;
                    }
                }
                sr.Close();
                return s_valuetext;
            }

            private static bool IsAlphaNumeric(String strToCheck)
            {
                Regex objAlphaNumericPattern = new Regex("[^a-zA-Z0-9]");
                return !objAlphaNumericPattern.IsMatch(strToCheck);
            }

        }
        public static string GetAppDataPath()
        {
            string AppDataPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData)) + mydirectory;
            return AppDataPath;
        }
        public static string GetModsPath()
        {
            string ModsPath = SettingsData.ModDataFolder + "mods\\";
            return ModsPath;
        }
        public static string GetTempPath()
        {
            string TempPath = SettingsData.ModDataFolder + "tmp\\";
            return TempPath;
        }
        public static string GetSCDPath()
        {
            string SCDPath = SettingsData.ModDataFolder + "scd\\";
            return SCDPath;
        }
        public static string GetDescPath()
        {
            string DescPath = SettingsData.ModDataFolder + "desc\\";
            return DescPath;
        }
        public static string GetSupCom2Path()
        {
            string rk_sc2dp = Registry.GetValue("HKEY_LOCAL_MACHINE\\SOFTWARE\\Valve\\Steam","InstallPath","SteamPath not found.").ToString();
            string p_sc2path = rk_sc2dp + "\\steamapps\\common\\supreme commander 2\\";

            if (!Directory.Exists(p_sc2path))
            {
                
                FolderBrowserDialog fbdialog = new FolderBrowserDialog();
                fbdialog.Description = "Please locate SC2 install path";
                if (fbdialog.ShowDialog() == DialogResult.OK)
                {
                    SettingsData.SC2Folder = fbdialog.SelectedPath+"\\";
                    if (Directory.Exists(SC2MMData.GetSupCom2DataPath()))
                    {
                        return fbdialog.SelectedPath + "\\";
                    }
                    else
                    {
                        MessageBox.Show("Couldn't find gamedata folder" + GetSupCom2DataPath() + ", please select the SC2 installation Folder", "Error");
                        GetSupCom2Path();
                    }
                }
                else
                {
                    SettingsData.SaveBlock = true;
                    if (File.Exists(SC2MMData.GetAppDataPath() + "settings.ini"))
                        File.Delete(SC2MMData.GetAppDataPath() + "settings.ini");
                    Application.Exit();
                }
            }
            return p_sc2path;
        }
        public static string GetSupCom2DataPath()
        {
            string p_gamedata = SettingsData.SC2Folder + "gamedata\\";
            return p_gamedata;
        }

    }
}
